// Initialising the amount of each type of component
const int LED_AMOUNT = 4;
const int LDR_COL_SIZE = 3;
const int LDR_ROW_SIZE = 2;
const int AMBIENT_PIN_SIZE = 2;

// Initialising arrays for each circuit
const int LED_PINS[LED_AMOUNT] = { 8, 9, 10, 11 };

// LDR Matrix
const int LDR_COL_PINS[LDR_COL_SIZE] = { 4, 5, 6 };
const int LDR_ROW_PINS[LDR_ROW_SIZE] = { A0, A1 };

// LDRs that should not be covered
const int AMBIENT_PINS[AMBIENT_PIN_SIZE] = { A2, A3 };

// Initialising key variables
int ldrMatrix[LDR_COL_SIZE][LDR_ROW_SIZE];
bool puzzleComplete = false;

void setup() {
  Serial.begin(9600);

  // Providing power to the LDRs on digital pins
  for (int i = 0; i < LDR_COL_SIZE; i++) {
    pinMode(LDR_COL_PINS[i], OUTPUT);
    digitalWrite(LDR_COL_PINS[i], HIGH);
  }
}

void loop() {
  if (!puzzleComplete) {
    delay(2000);
    // Read the LDR values
    createMatrix();

    // Chck main if main LDRs are covered and ambient LDRs are uncovered
    puzzleComplete = (readAmbience() && readMatrix());
  }

  if (puzzleComplete) {
    // Make LEDs light up in sequence and repeat
    for (int i = 0; i < LED_AMOUNT; i++) {
      digitalWrite(LED_PINS[i], HIGH);
      delay(500);
    }
    delay(3000);
    for (int i = 0; i < LED_AMOUNT; i++) {
      digitalWrite(LED_PINS[i], LOW);
    }
    delay(500);
  }
}

// Function to create a matrix of the values of the main LDRs
void createMatrix() {
  for (int i = 0; i < LDR_COL_SIZE; i++) {
    digitalWrite(LDR_COL_PINS[i], HIGH);

    for (int j = 0; j < LDR_ROW_SIZE; j++) {
      ldrMatrix[i][j] = analogRead(LDR_ROW_PINS[j]);
    }
    digitalWrite(LDR_COL_PINS[i], LOW);
  }
}

// Function to check if all main LDRs are covered
bool readMatrix() {
  for (int i = 0; i < LDR_COL_SIZE; i++) {
    for (int j = 0; j < LDR_ROW_SIZE; j++) {
      if (ldrMatrix[i][j] > 50) {
        Serial.println("LEDs do not activate this time. Matrix.\n");
        return false;
      }
    }
  }
  return true;
}

// Function to check if all ambient LDRs are uncovered
bool readAmbience() {
  for (int i = 0; i < AMBIENT_PIN_SIZE; i++) {
    // LEDs will not light up if either ambient pin is covered
    if (analogRead(AMBIENT_PINS[i]) < 30) {
      Serial.println("LEDs do not activate this time. Ambient.\n");
      return false;
    }
  }
  return true;
}